
/******************************************************************
Done by Francois Braud
******************************************************************/

#ifdef _MSC_VER
	#include <al.h>
	#include <alut.h>
#else
	#include <AL/al.h>
	#include <AL/alut.h>
#endif

#include <cstdio>
#include <iostream>
#include <vector>

#include <time.h>
#ifdef WIN32
	#include <Windows.h>
#else
    #include <sys/time.h>
#endif

#include "Sound_OpenAL.h"

static void sleepMs(unsigned int miliseconds)
{
#ifdef WIN32
  Sleep (miliseconds);
#else
  usleep (miliseconds * 1000);
#endif
}

//------------------------------------------------------------------------------
void Sound_OpenAL::play(bool blockUntilEnd)
{
    // Finally, play the sound!!!
    alSourcePlay(sourceID);
	//std::cout << "play the sound" << std::endl;

    if (blockUntilEnd)
    {
        ALint state;// The state of the sound source

        do {
            // Query the state of the souce
            alGetSourcei(sourceID, AL_SOURCE_STATE, &state);
            sleepMs(10);
        } while (state != AL_STOPPED);
    }
}
//------------------------------------------------------------------------------
Sound_OpenAL::~Sound_OpenAL()
{
    // Clean up sound buffer
    alDeleteBuffers(1, &bufferID);
    // Clean up sound source
    alDeleteSources(1, &sourceID);
}
//------------------------------------------------------------------------------
bool Sound_OpenAL::isPlaying() const
{
    ALint state;
    alGetSourcei(sourceID, AL_SOURCE_STATE, &state);
    return state != AL_STOPPED && state != AL_INITIAL;
}
//------------------------------------------------------------------------------
void Sound_OpenAL::setVolume(unsigned char vol)
{
    alSource3f(sourceID, AL_POSITION, 2000000+255.f*100 - (float)vol*100, 0.0f, 0.0f);
}
//------------------------------------------------------------------------------
Sound_OpenAL* Sound_OpenAL::loadSoundFromFile(const char* fileName)
{
    ALuint bufferID;            // The OpenAL sound buffer ID
    ALuint sourceID;            // The OpenAL sound source
    std::vector <char> bufferData;// The sound buffer data from file

    // Create sound buffer and source
    //alGenBuffers(1, &bufferID);
    alGenSources(1, &sourceID);

    // Set the source and listener to the same location
    alListener3f(AL_POSITION, 0.0f, 0.0f, 0.0f);
    alSource3f(sourceID, AL_POSITION, 0.0f, 0.0f, 0.0f);

    bufferID = alutCreateBufferFromFile(fileName);
	
    // Attach sound buffer to source
    alSourcei(sourceID, AL_BUFFER, bufferID);

	std::cout << "Loaded sound: " << fileName << ";" << bufferID << ";" << sourceID << std::endl;

    return new Sound_OpenAL(bufferID, sourceID);
}
//------------------------------------------------------------------------------
void initSound_OpenAL(int argc, char *argv[])
{
    // Initialize the OpenAL library
    //ALboolean ret =
    alutInit(&argc, argv);
    //Assert(ret == AL_TRUE);
}
//------------------------------------------------------------------------------
void deinitSound_OpenAL()
{
    // Clean up the OpenAL library
    alutExit();
}
//------------------------------------------------------------------------------
