#include <string.h>
#include <stdio.h>
#include <iostream>
#include "util.h"

#pragma comment(lib, "Irrlicht.lib")
//project options :
//links : -lIrrlicht -ljpeg -lz -lm
//include dir : F:\progs\Dev-Cpp\include\irrlicht\
//lib dir : F:\progs\Dev-Cpp\lib\irrlicht\

#include "const.h"
#include "settings.h"
#include "world.h"
#include "soundbank.h"

#define HELP_MESSAGE "The only option is --setup. Read the readme.html for more help.\n"

#define SETUP_ENGLISH_PHASE_1 "\nPlease select the difficulty level :\n\n"
#define SETUP_FRENCH_PHASE_1 "\nVeuillez choisir le niveau de difficult.\n\n"
#define SETUP_ENGLISH_PHASE_2 "\nPlease select the screen mode you want to use :\n(i) is advised.\nIf the game is too slow, reduce screen mode or select a windowed mode.\n\n"
#define SETUP_FRENCH_PHASE_2 "\nVeuillez choisir la resolution d'ecran que vous voulez utiliser :\n(i) est conseille.\nSi le jeu rame, reduisez la resolution et/ou selectionnez un mode fenetre (windowed).\n\n"
#define SETUP_ENGLISH_PHASE_3 "\nPlease select the driver you want :\n(a) is advised.\n\n"
#define SETUP_FRENCH_PHASE_3 "\nVeuillez choisir le pilote voulu :\n(a) est conseille.\n\n"
#define SETUP_ENGLISH_PHASE_4 "\nPlease select the mouse rotation speed you want :\n(b) is advised.\n\n"
#define SETUP_FRENCH_PHASE_4 "\nVeuillez choisir la vitesse de rotation de la souris :\n(b) est conseille.\n\n"
#define SETUP_ENGLISH_PHASE_5 "\nPlease select the display bits you want :\n(It will change something only if you selected a fullscreen mode)\n(c) is advised.\n\n"
#define SETUP_FRENCH_PHASE_5 "\nVeuillez choisir la qualite de couleur :\n(Cela ne changera quelque chose que si le jeu est en plein ecran)\n(c) est conseille.\n\n"
#define SETUP_ENGLISH_PHASE_6 "\nPlease select if you want to see the cubes move. Select (b) if you experiment problems like collision error or segmentation faults.\n\n"
#define SETUP_FRENCH_PHASE_6 "\nVeuillez choisir si vous voulez que les cubes se dplacent. Choisissez (b) si vous rencontrez des problmes de collision ou des erreurs de segmentation.\n\n"


#define ENGLISH_TEXT_1 ". Press Return Key to play."
#define FRENCH_TEXT_1 ". Appuyez sur Entree pour jouer."
#define ENGLISH_TEXT_2 L"To quit the game, press ESCAPE. To continue, press Return."
#define FRENCH_TEXT_2 L"Pour quitter le jeu, appuyez sur ECHAP. Pour continuer, pressez Entree."
#define ENGLISH_TEXT_3 L"You finished the game ! Press Return Key to exit."
#define FRENCH_TEXT_3 L"Vous avez fini le jeu ! Appuyez sur Entree pour quitter."
#define ENGLISH_TEXT_4 L"You lost all your lifes ! Press Return Key to retry."
#define FRENCH_TEXT_4 L"Vous avez perdu toutes vos vies ! Appuyez sur Entree pour Reessayer."

#define INITIAL_LEVEL 1
#define INITIAL_LIFE 3

static int level=INITIAL_LEVEL;
static int life=INITIAL_LIFE;

// la classe de traintement de messages.
// elle doit hriter de IEventReceiver
// et implmenter sa mthode OnEvent
class MyEventReceiver : public IEventReceiver
{
public:
    // c'est ici qu'on recupere les evenements
	virtual bool OnEvent(const SEvent& event)
	{

	    if(event.EventType == EET_KEY_INPUT_EVENT) 
	    {
       	    if(event.KeyInput.PressedDown)
            {
                keys[event.KeyInput.Key] = true;
            }
            else if(!event.KeyInput.PressedDown)
            {
                keys[event.KeyInput.Key] = false;
            }
        }    
       else if(event.EventType == EET_MOUSE_INPUT_EVENT)
        {                 
            if(event.MouseInput.Event == EMIE_LMOUSE_PRESSED_DOWN)
                mouseLeft = true;
            if(event.MouseInput.Event == EMIE_RMOUSE_PRESSED_DOWN)
                mouseRight = true;
            if(event.MouseInput.Event == EMIE_MMOUSE_PRESSED_DOWN)
                mouseMiddle = true;
            
            if(event.MouseInput.Event == EMIE_LMOUSE_LEFT_UP)
                mouseLeft = false;
            if(event.MouseInput.Event == EMIE_RMOUSE_LEFT_UP)
                mouseRight = false;
            if(event.MouseInput.Event == EMIE_MMOUSE_LEFT_UP)
                mouseMiddle = false;
  
            if(event.MouseInput.Event == EMIE_MOUSE_WHEEL)
            {
                if (event.MouseInput.Wheel>0)
                    mouseWheel++;
                else
                    mouseWheel--;
           }
		    return false;
        }

        
        // retourner false veut dire qu'on n'a pas trait l'vnement
        // et si on passe ici, c'est qu'on a reu un vnement qui ne nous
        // intressait pas, donc on l'a pas trait donc on retourne false
		return false;
	}
};


static void getConsoleChoice(void)
{
    char i;
    
	printf("\n#################\n");
	printf("Application setup\n");
	printf("#################\n");

	printf("\nPlease select your language :\n\n");
	printf(" (a) English\n (b) Francais\n"\
        " (q) quit setup (and save)\n"\
		" (otherKey) skip this choice\n\n");
	
	std::cin >> i;	
	
	if (i=='a')
	     settings_SetValue("is_french",false);
	else if (i=='b')
		settings_SetValue("is_french",true);
	else if (i=='q')
		return;

    	bool is_french=settings_GetValue("is_french");

	//phase 1
	printf(is_french?SETUP_FRENCH_PHASE_1:SETUP_ENGLISH_PHASE_1);
    printf(is_french?" (a) facile\n (b) moyen\n (c) avanc\n (d) difficile\n (e) transcendant\n\n":" (a) easy\n (b) medium\n (c) advanced\n (d) difficult\n (e) transcendant\n");
	
 	std::cin >> i;

	switch(i)
	{
		case 'a': settings_SetValue("difficulty_level",1);settings_SetValue("time_counter_speed",800);break;
		case 'b': settings_SetValue("difficulty_level",2);settings_SetValue("time_counter_speed",350);break;
		case 'c': settings_SetValue("difficulty_level",3.5);settings_SetValue("time_counter_speed",180);break;
        case 'd': settings_SetValue("difficulty_level",5);settings_SetValue("time_counter_speed",100);break;
        case 'e': settings_SetValue("difficulty_level",8);settings_SetValue("time_counter_speed",60);break;
        case 'q':return;
	}	
    
	//phase 2
	printf(is_french?SETUP_FRENCH_PHASE_2:SETUP_ENGLISH_PHASE_2);
	printf(" (a) 400x300 windowed\n (b) 640x480 windowed\n (c) 800x600 windowed\n"\
		" (d) 1024x768 windowed\n (e) 1152x864 windowed\n (f) 1280x1024 windowed\n"\
		" (g) 640x480 fullscreen\n (h) 800x600 fullscreen\n (i) 1024x768 fullscreen\n"\
		" (j) 1152x864 fullscreen\n (k) 1280x960 fullscreen\n (l) 1280x1024 fullscreen\n\n");
	
 	std::cin >> i;
 	
 	int window_size_x, window_size_y;
	bool is_fullscreen;
    
	switch(i)
	{
		case 'a': window_size_x=400;window_size_y=300;is_fullscreen=false;break;
		case 'b': window_size_x=640;window_size_y=480;is_fullscreen=false;break;
		case 'c': window_size_x=800;window_size_y=600;is_fullscreen=false;break;
		case 'd': window_size_x=1024;window_size_y=768;is_fullscreen=false;break;
		case 'e': window_size_x=1152;window_size_y=864;is_fullscreen=false;break;
		case 'f': window_size_x=1280;window_size_y=1024;is_fullscreen=false;break;
		case 'g': window_size_x=640;window_size_y=480;is_fullscreen=true;break;
		case 'h': window_size_x=800;window_size_y=600;is_fullscreen=true;break;
		case 'i': window_size_x=1024;window_size_y=768;is_fullscreen=true;break;
		case 'j': window_size_x=1152;window_size_y=864;is_fullscreen=true;break;
		case 'k': window_size_x=1280;window_size_y=960;is_fullscreen=true;break;
		case 'l': window_size_x=1280;window_size_y=1024;is_fullscreen=true;break;
        case 'q':return;
	}
	if (i=='a' || i=='b' || i=='c' || i=='d' || i=='e' || i=='f' || i=='g' || 
	 i=='h' || i=='i' || i=='j' || i=='k')
	 {
		 settings_SetValue("window_size_x",window_size_x);
       settings_SetValue("window_size_y",window_size_y);
           settings_SetValue("is_fullscreen",is_fullscreen);
    	}    

   	//phase 3
	printf(is_french?SETUP_FRENCH_PHASE_3:SETUP_ENGLISH_PHASE_3);
	printf(" (a) Direct3D 9.0c (Windows) | OpenGL 1.2 (Linux)\n (b) OpenGL 1.2 \n"\
		" (c) Software Renderer (not advised/deconseille)\n\n");
	
	while(1)
	{
		i=fgetc(stdin);
		if (i!='\0')
			break;
	}
	
	std::cin >> i;
	
	switch(i)
	{
		case 'a': settings_SetValue("driver_type",6);break;
		case 'b': settings_SetValue("driver_type",util_GetDriverNum(video::EDT_OPENGL));break;
		case 'c': settings_SetValue("driver_type",util_GetDriverNum(video::EDT_SOFTWARE));break;
        case 'q':return;
	}

	//phase 4
	printf(is_french?SETUP_FRENCH_PHASE_4:SETUP_ENGLISH_PHASE_4);
	printf(" (a) 2\n (b) 4\n (c) 8\n\n");
	
 	std::cin >> i;

	switch(i)
	{
		case 'a':settings_SetValue("mouse_rotation_speed",2);break;
		case 'b':settings_SetValue("mouse_rotation_speed",4);break;
		case 'c':settings_SetValue("mouse_rotation_speed",8);break;
        case 'q':return;
	}	
	
	//phase 5
	printf(is_french?SETUP_FRENCH_PHASE_5:SETUP_ENGLISH_PHASE_5);
    	printf(" (a) 8\n (b) 16\n (c) 32\n\n");
	
 	std::cin >> i;

	switch(i)
	{
		case 'a': settings_SetValue("display_bits",8);break;
		case 'b': settings_SetValue("display_bits",16);break;
		case 'c': settings_SetValue("display_bits",32);break;
        case 'q':return;
	}	
	
	//phase 6
	printf(is_french?SETUP_FRENCH_PHASE_6:SETUP_ENGLISH_PHASE_6);
	printf(" (a) on\n (b) off\n\n");
	
 	std::cin >> i;

	switch(i)
	{
		case 'a':settings_SetValue("moving_cubes",1.0);break;
		case 'b':settings_SetValue("moving_cubes",0.0);;break;
        case 'q':return;
	}	
}


int main(int argc, char *argv[])
{   
	util_InitNullDevice();

	settings_Init();
    
	settings_ReadXMLSettingsFile(CONFIG_FILE_PATH);

	if (argc>1)
	{
	        if (strcmp(argv[1],"--setup")==0)
        	{
	            getConsoleChoice();
				printf("\nWriting on disk...\n");
	            settings_WriteXMLSettingsFile(CONFIG_FILE_PATH);
				printf("\nLaunching game, please wait...\n");
	        }
		else
	        {
	            fprintf(stderr,HELP_MESSAGE);
	            return EXIT_FAILURE;
	        }
	}    
    
	util_DropDevice();
	
	MyEventReceiver receiver;
	util_Init(&receiver,
                (int)settings_GetValue("window_size_x"),
                (int)settings_GetValue("window_size_y"),
                (bool)settings_GetValue("is_fullscreen"),
                (int)settings_GetValue("display_bits"),
                util_GetDriverType((int)settings_GetValue("driver_type"))
                );

    //charge le son
    soundbank_init(argc, argv);
    soundbank_loadSound("data/jump.wav");
    soundbank_loadSound("data/walk.wav");
    soundbank_loadSound("data/alarm.wav");

    mouseWheel=15;
    
	bool is_french=settings_GetValue("is_french");

    	//recuperer la police
	IGUISkin* skin = env->getSkin();
	IGUIFont* font = env->getFont("data/fontcourier.bmp");
	if (font)
		skin->setFont(font);
    
	// charger l'image de fond du texte en bas de l'ecran
	ITexture* texture = driver->getTexture("data/textback.bmp");
	IGUIImage* img = env->addImage(rect<int>(0,util_GetWindowSizeY()-30,640,util_GetWindowSizeY()));
	img->setImage(texture);
    
    
	//creer le texte en bas de l'ecran
	core::stringw first_screen_text=L"";
	first_screen_text += APPLICATION_TITLE;
	first_screen_text += is_french?FRENCH_TEXT_1:ENGLISH_TEXT_1;
	IGUIStaticText *static_text=env->addStaticText(first_screen_text.c_str(),
		rect<s32>(10,util_GetWindowSizeY()-25,600,util_GetWindowSizeY()),
		false, // border?
		true // wordwrap?
		);
	
	//attendre l'appui de Entree
    	util_WaitForReturnKey(APPLICATION_TITLE);
    
	int status_game;
	bool key_tab_wait=false;

    core::stringw str = L"";
    str += APPLICATION_TITLE;
    str += " (";
	str += driver->getName();
	str += ")";
	device->setWindowCaption(str.c_str());
	
	//boucle refaite a chaque nouveau niveau
	while(level<=LEVEL_MAX)
	{
	    if (life<=0)
	    {
	         life=INITIAL_LIFE;
	         level=INITIAL_LEVEL;
	         core::stringw str=is_french?FRENCH_TEXT_4:ENGLISH_TEXT_4;
   	         static_text->setText(str.c_str());
   	         util_WaitForReturnKey(APPLICATION_TITLE);
   	    }    
	    
	    //chargement du niveau
        world_Load(level,static_text);
        
        //boucle principale
	while(true)
	{
            if (!device->run() || !driver)
    		{
    		    world_Destroy();
    		    util_QuitApplication();
    		}    
            
			status_game = world_Play();
            
            //affichage du texte en bas
            core::stringw str=L"";
            str+= is_french?"vie=":"life=";
            str+= life;
            str+= is_french?", niveau=":", level=";
            str+= level;
            str+= ", FPS=";
            str+= util_GetFPS();
            str+= ", distance=";
            str+= status_game;
            if (world_GetRemainingTime()!=-1000)
            {
                str+= is_french?", temps=":", time=";
                str+= (int)(world_GetRemainingTime()/1000);
                if (world_GetRemainingTime()<5000)
                {
                    str+=" !!!";
                    soundbank_playSound(2);
                }
            }
            static_text->setText(str.c_str());
            
			// commencer la scne en vidant l'cran avec du gris
			// et en remettant le zbuffer a zro
			driver->beginScene(true, true, video::SColor(0,0,0,0));
		    
			smgr->drawAll();
			world_CustomDraw();
			env->drawAll();
			
			// fin du rendu
			driver->endScene();
			
			util_CalcFPS();

            if (status_game==STATUS_GAME_PLAYER_LOSE || (world_GetRemainingTime()!=-1000 && world_GetRemainingTime()<0))
            {
                life--;
                break;
            }
            else if (status_game==STATUS_GAME_PLAYER_WIN)
            {
                life++;
                level++;
                break;
            }
            else if (keys[KEY_TAB] && key_tab_wait==false)
            {
		        util_Fade(video::SColor(0,0,0,0),1000);
                life--;
                key_tab_wait=true;
                break;
            }
            
            if (!keys[KEY_TAB] && key_tab_wait==true)
                key_tab_wait=false;
            
            if (keys[KEY_ESCAPE])
            {
	           while(keys[KEY_ESCAPE])
                 if (!device->run() || !driver)
       		     {
                     world_Destroy();
   		             settings_Destroy();
		             util_QuitApplication();
  		         }    
   	           core::stringw str=is_french?FRENCH_TEXT_2:ENGLISH_TEXT_2;
	           static_text->setText(str.c_str());
	           if (!util_WaitForReturnKey(APPLICATION_TITLE))
       		     {
                     world_Destroy();
   		             settings_Destroy();
		             util_QuitApplication();
  		         }    
            }  

            if (keys[KEY_KEY_N])
                soundbank_setMute(true);
            if (keys[KEY_KEY_B])
                soundbank_setMute(false);
        } 
           
        //liberation des ressources du niveau
	world_Destroy();
    }	
    
    if (level>LEVEL_MAX)
    {
        core::stringw str=is_french?FRENCH_TEXT_3:ENGLISH_TEXT_3;
        static_text->setText(str.c_str());
    }
    
    //attendre l'appui de Entree
    util_WaitForReturnKey(APPLICATION_TITLE);
    
    //au revoir
    settings_Destroy();
    util_QuitApplication();
	
    return EXIT_FAILURE;
}
