#include <math.h>
#ifndef M_PI
    #define M_PI 3.14159265358979323846
#endif
#include "util.h"

#include "object.h"
#include "player.h"
#include "tableworld.h"
#include "const.h"

Object::Object(const char* mesh_path,const char *texture_path,double radius_,double height_,bool is_visible)
{
    radius=radius_;
    height=height_;
    
    if (mesh_path == NULL)
        util_PrintCriticalError("Object don't has any mesh");

  	obj_mesh = smgr->getMesh(util_TestExistFile(mesh_path));
   	// rcuprer le mesh statique
    obj = smgr->addMeshSceneNode(obj_mesh->getMesh(0));
    // lui assigner la texture qu'on charge  la vole
    if (texture_path != NULL)
        obj->setMaterialTexture(0, driver->getTexture(util_TestExistFile(texture_path)));
    // supprimer la gestion de l'clairage sur le cube
  	obj->setMaterialFlag(EMF_LIGHTING, false);
  	
  	obj->setScale(vector3df(0.1,0.1,0.1));
    	
   	obj->setPosition(tableworld_GetRandomPosition());

	obj->setVisible(is_visible);
}

void Object::Move(double mvt)
{
    obj->setPosition(core::vector3df(
        obj->getPosition().X-cos(obj->getRotation().Y*M_PI/180)*mvt,
        obj->getPosition().Y,
        obj->getPosition().Z+sin(obj->getRotation().Y*M_PI/180)*mvt
        ));
}    

void Object::MoveDown(double mvt)
{
    obj->setPosition(core::vector3df(
        obj->getPosition().X,
        obj->getPosition().Y+mvt,
        obj->getPosition().Z
        ));
}    

void Object::MoveRight(double mvt)
{
    obj->setPosition(core::vector3df(
        obj->getPosition().X-cos((obj->getRotation().Y+90)*M_PI/180)*mvt,
        obj->getPosition().Y,
        obj->getPosition().Z+sin((obj->getRotation().Y+90)*M_PI/180)*mvt
        ));
}    

void Object::Rotate(double rot_y)
{
    obj->setRotation(core::vector3df(obj->getRotation().X,obj->getRotation().Y+rot_y,obj->getRotation().Z));
}    

bool Object::IsColliding()
{
    if (tableworld_GetWithPos(obj->getPosition())) return true;
    else if (tableworld_GetWithPos(core::vector3df(
            obj->getPosition().X+radius,
            obj->getPosition().Y,
            obj->getPosition().Z))) return true;
    else if (tableworld_GetWithPos(core::vector3df(
            obj->getPosition().X-radius,
            obj->getPosition().Y,
            obj->getPosition().Z))) return true;
    else if (tableworld_GetWithPos(core::vector3df(
            obj->getPosition().X,
            obj->getPosition().Y+height,
            obj->getPosition().Z))) return true;
    else if (tableworld_GetWithPos(core::vector3df(
            obj->getPosition().X,
            obj->getPosition().Y-height,
            obj->getPosition().Z))) return true;
    else if (tableworld_GetWithPos(core::vector3df(
            obj->getPosition().X,
            obj->getPosition().Y,
            obj->getPosition().Z+radius))) return true;
    else if (tableworld_GetWithPos(core::vector3df(
            obj->getPosition().X,
            obj->getPosition().Y,
            obj->getPosition().Z-radius))) return true;
    
    return false;
}   

vector3df Object::GetPosition()
{
    return obj->getPosition();
}    

vector3df Object::GetRotation()
{
    return obj->getRotation();
}    
