#include <math.h>
#include "util.h"

#include "player.h"
#include "const.h"
#include "settings.h"

#include "soundbank.h"

#define PLAYER_SPEED 2.0
#define PLAYER_JUMP_CAPACITY 200.0
#define GRAVITY 400.0

Player::Player(const char* mesh_path,const char *texture_path)
:speed_y(0),no_jump(true),camera_angle_x(20),is_distance_status_3D(true)
{
    is_subjective=(bool)settings_GetValue("is_subjective");
    mouse_rotation_speed=settings_GetValue("mouse_rotation_speed");
    keys_rotation_speed=settings_GetValue("keys_rotation_speed");
    
    obj=new Object(mesh_path,texture_path,3.0,6.0,!is_subjective);
    cursor=device->getCursorControl();
    window_size_x=util_GetWindowSizeX();
    window_size_y=util_GetWindowSizeY();
    
}   

Player::~Player()
{
    delete obj;
}

int Player::Play(ICameraSceneNode* camera,ISceneNode *objective)
{
    int fps = util_GetFPS();
    
    double mvt=(double)((double)30/(double)fps);
    if (keys[KEY_LEFT] || keys[KEY_KEY_K])
        obj->Rotate(-mvt*keys_rotation_speed);
    if (keys[KEY_RIGHT] || keys[KEY_KEY_M])
        obj->Rotate(mvt*keys_rotation_speed);
    if (keys[KEY_UP] || keys[KEY_KEY_R] || mouseLeft)
    {
        
        obj->Move(mvt*PLAYER_SPEED);
        if (obj->IsColliding())
            obj->Move(-mvt*PLAYER_SPEED);
        else
            soundbank_playSound(1);
    }    
    if (keys[KEY_DOWN] || keys[KEY_KEY_F] || mouseMiddle)
    {
        obj->Move(-mvt*PLAYER_SPEED);
        if (obj->IsColliding())
            obj->Move(mvt*PLAYER_SPEED);
        else
            soundbank_playSound(1);
    }   
    
    if (keys[KEY_KEY_D])
    {
        obj->MoveRight(-2*mvt*PLAYER_SPEED/3);
        if (obj->IsColliding())
            obj->MoveRight(2*mvt*PLAYER_SPEED/3);
        else
            soundbank_playSound(1);
    }    
    
    if (keys[KEY_KEY_G])
    {
        obj->MoveRight(2*mvt*PLAYER_SPEED/3);
        if (obj->IsColliding())
            obj->MoveRight(-2*mvt*PLAYER_SPEED/3);
        else
            soundbank_playSound(1);
    }    
    if (keys[KEY_KEY_O] || keys[KEY_PRIOR])
        camera_angle_x-=keys_rotation_speed*(double)((double)30/(double)fps);
    if (keys[KEY_KEY_L] || keys[KEY_NEXT])
        camera_angle_x+=keys_rotation_speed*(double)((double)30/(double)fps);
    
    position2d<s32> pos_cursor=cursor->getPosition();
    if (pos_cursor.X<(window_size_x/2)-5)
        obj->Rotate(-(mouse_rotation_speed*1.5*((double)(window_size_x-pos_cursor.X-5))/fps)/5);
    if (pos_cursor.X>(window_size_x/2)+5)
        obj->Rotate(-(mouse_rotation_speed*1.5*((double)(pos_cursor.X-window_size_x-5))/fps)/5);
    if (pos_cursor.Y<(window_size_y/2)-5)
        camera_angle_x-=(mouse_rotation_speed*((double)(window_size_y-pos_cursor.Y-5))/fps)/5;
    if (pos_cursor.Y>(window_size_y/2)+5)
        camera_angle_x-=(mouse_rotation_speed*((double)(pos_cursor.Y-window_size_y-5))/fps)/5;
    cursor->setPosition(position2d<s32>((int)window_size_x/2,(int)window_size_y/2));
    
    speed_y-=GRAVITY/(double)fps;
    obj->MoveDown(speed_y/(double)fps);
    if (obj->IsColliding())
    {
        obj->MoveDown(-speed_y/(double)fps);
        if (speed_y<=0)
        no_jump=false;
        speed_y=0;
    }    
    
    if ((keys[KEY_SHIFT] || keys[KEY_SPACE] || mouseRight)&& no_jump==false)
    {
        soundbank_playSound(0);
        speed_y=PLAYER_JUMP_CAPACITY;
        no_jump=true;
    }    
    
    if (!is_subjective)
    {
        camera->setPosition(vector3df(obj->GetPosition().X,obj->GetPosition().Y+DISTANCE_CAMERA,obj->GetPosition().Z));
        
        irr::core::matrix4 m;
        m.setRotationDegrees(vector3df(90,obj->GetRotation().Y-90,0));
         
        irr::core::vector3df v = irr::core::vector3df(0, 0, DISTANCE_CAMERA); //target
        irr::core::vector3df u = irr::core::vector3df(0, 1, 0); //up vector
        m.transformVect(v);
        v += vector3df(obj->GetPosition().X,obj->GetPosition().Y+DISTANCE_CAMERA,obj->GetPosition().Z);
        m.transformVect(u);
        
        camera->setUpVector(u);
        camera->setTarget(v);
    }
    else    
    {
        camera->setPosition(obj->GetPosition());
        
        irr::core::matrix4 m;
        m.setRotationDegrees(vector3df(camera_angle_x,obj->GetRotation().Y-90,0));
         
        irr::core::vector3df v = irr::core::vector3df(0, 0, 100); //target
        irr::core::vector3df u = irr::core::vector3df(0, 1, 0); //up vector
        m.transformVect(v);
        v += vector3df(obj->GetPosition());
        m.transformVect(u);
    
        camera->setUpVector(u);
        camera->setTarget(v);
    }  

/*
	printf("player->%d %d %d \n",(int)(obj->GetPosition().X/CUBE_SIZE),(int)(obj->GetPosition().Y/CUBE_SIZE),(int)(obj->GetPosition().Z/CUBE_SIZE));
	printf("objective->%d %d %d \n",(int)(objective->getPosition().X/CUBE_SIZE),(int)(objective->getPosition().Y/CUBE_SIZE),(int)(objective->getPosition().Z/CUBE_SIZE));
*/
	if (keys[KEY_F3])
		is_distance_status_3D=true;
	if (keys[KEY_F2])
		is_distance_status_3D=false;

	if ((int)(obj->GetPosition().X/CUBE_SIZE)==(int)(objective->getPosition().X/CUBE_SIZE)
	        && (int)(obj->GetPosition().Y/CUBE_SIZE)==(int)(objective->getPosition().Y/CUBE_SIZE)
	        && (int)(obj->GetPosition().Z/CUBE_SIZE)==(int)(objective->getPosition().Z/CUBE_SIZE))
	{
		util_Fade(video::SColor(0,255,255,255),1000);
		return STATUS_GAME_PLAYER_WIN;
	}
	else if (obj->GetPosition().Y < -50)
	{
		util_Fade(video::SColor(0,0,0,0),1000);
	        return STATUS_GAME_PLAYER_LOSE;
	}
	else 
	{
		if (is_distance_status_3D)
		        return GetDistanceFromObjective(objective);
		else
		{
			double dist_x=obj->GetPosition().X-objective->getPosition().X;
			double dist_z=obj->GetPosition().Z-objective->getPosition().Z;
			return (int)(sqrt(dist_x*dist_x+dist_z*dist_z));
		}
	}
}   


vector3df Player::GetPosition()
{
	return obj->GetPosition();
}

int Player::GetDistanceFromObjective(ISceneNode *objective)
{
    return (int)obj->GetPosition().getDistanceFrom(objective->getPosition());
}
