#include "util.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "settings.h"

struct str_item{
  char *name;
  double value;
};
typedef struct str_item struct_item;

static str_item *table_settings;
static int nb_settings=0;
static int size_table_settings=100;

const char* tmptxt = "<?xml version=\"1.0\"?>\r\n\t<config>\r\n\t\t"\
	"<!-- This is the config file of the application -->\r\n\t\t<all_settings \r\n";

const char* tmptxt3 = "\t\t/>\r\n\t</config>\r\n";
/*
S'il n'y a pas de fichier config charg, il faut alors inclure a dans main :

    settings_AddItem("size_x_world",7.0);
    settings_AddItem("size_y_world",7.0);
    settings_AddItem("size_z_world",7.0);
    settings_AddItem("probability_cube",0.5);//0.7);
    settings_AddItem("is_subjective",true);
    settings_AddItem("is_fog_enable",false);
    settings_AddItem("is_glasses_world",false);
    settings_AddItem("mouse_rotation_speed",2.0);
    settings_AddItem("keys_rotation_speed,"10.0");
*/



void settings_AddItem(const char* name,double initial_value)
{
    //printf("Add setting -> %s=%lf\n",name,initial_value);
    table_settings[nb_settings].name=new char[strlen(name)+1];
    strncpy(table_settings[nb_settings].name,name,strlen(name)+1);
    table_settings[nb_settings].value=initial_value;
    nb_settings++;
//for (int i=0;i<nb_settings;i++)
//   printf("--->%d %s\n",strlen(table_settings[i].name),table_settings[i].name);

   //reallocation si trop petit
    if (nb_settings==size_table_settings)
    {
        size_table_settings*=2;
//        item *tmp_table=new struct_item[size_table_settings];
//        memcpy (tmp_table,table_settings,size_table_settings*sizeof(struct item));
//        delete [] table_settings;
//        table_settings=tmp_table;
        table_settings=(str_item*)realloc(table_settings,size_table_settings*sizeof(struct_item));
    }  
}    

void settings_Init(void)
{
    table_settings=(str_item*)malloc(size_table_settings*sizeof(struct_item));
}    

double settings_GetValue(const char *name_setting)
{
  for (int i=0;i<nb_settings;i++)
    {
      //printf("->%d %d %s\n",strlen(table_settings[i].name),strlen(name_setting),table_settings[i].name);
        if (strcmp(name_setting,table_settings[i].name)==0)
	  return table_settings[i].value;
    }
    fprintf(stderr,"%s :",name_setting);
    util_PrintCriticalError("setting unknown");
    return 0;
}    

void settings_ReadXMLSettingsFile(const char *config_file)
{
    printf("Getting settings from file %s\n",config_file);
    io::IXMLReader* xml = device->getFileSystem()->createXMLReader(config_file);
    
    if (xml==NULL)
    {
         fprintf(stderr,"%s :",config_file);
         util_PrintCriticalError("unable to open (readonly) settings file");
    }    
    
    while(xml && xml->read())
        {
                switch(xml->getNodeType())
                {
                    case io::EXN_ELEMENT:
                        if (core::stringw("all_settings") == xml->getNodeName())
                        {
                            for (int i=0;i<xml->getAttributeCount();i++)
                            {
                                core::stringw str=xml->getAttributeName(i);
                                char *str2=new char[str.size()+1];
                                for (int j=0;j<str.size()+1;j++)
                                     str2[j]=(char)str[j];
                                settings_AddItem(str2,
                                    xml->getAttributeValueAsFloat(xml->getAttributeName(i)));
                                delete [] str2;

                                //printf("%s %lf\n",str2,xml->getAttributeValueAsFloat(xml->getAttributeName(i)));
                            }    
                        }    
                        break;
                }
        }       
    
    if (xml)
        xml->drop();
}    

void settings_Destroy(void)
{
    free(table_settings);
}    


void settings_SetValue(const char *name_setting,double new_value)
{
    for (int i=0;i<nb_settings;i++)
        if (strcmp(name_setting,table_settings[i].name)==0)
        {
             table_settings[i].value=new_value;
             return;
        }    
    
    fprintf(stderr,"%s :",name_setting);
    util_PrintCriticalError("setting unknown");
}    



void settings_WriteXMLSettingsFile(const char *config_file)
{
	printf("Writing settings in file %s\n",config_file);
	io::IWriteFile *xmlfile=device->getFileSystem()->createAndWriteFile(config_file,false);
    
    if (xmlfile)
    {
        xmlfile->write(tmptxt,strlen(tmptxt));
        //delete [] tmptxt;
       	
        for (int i=0;i<nb_settings;i++)
        {
            char* tmptxt2 = new char[100+strlen(table_settings[i].name)];
            sprintf(tmptxt2,"\t\t\t%s=\"%lf\"\r\n",table_settings[i].name,table_settings[i].value,(char)13,(char)10);
            xmlfile->write(tmptxt2,strlen(tmptxt2));
            delete [] tmptxt2;
        }   
        
        xmlfile->write(tmptxt3,strlen(tmptxt3));
           
        xmlfile->drop();
    }    
    else
    {
         fprintf(stderr,"%s :",config_file);
         util_PrintError("unable to open (writeonly) settings file");        
    }    

}    

