/*********************************************************
soundbank.cpp
manage sounds, using Audiere
This file is part of the IRRSIMPLE project
made by phrounz 
Require : system, Audiere library
*********************************************************/

#include <iostream>
#define ENABLE_SOUND

#ifdef ENABLE_SOUND
	#include <vector>
	#include "Sound_OpenAL.h"
	std::vector<Sound_OpenAL*> allSounds;
#endif

bool isMute = false;

void soundbank_init(int argc, char* argv[])
{
	initSound_OpenAL(argc, argv);
}

void soundbank_loadSound(const char *pathname)
{
#ifdef ENABLE_SOUND
	Sound_OpenAL* newsound = Sound_OpenAL::loadSoundFromFile(pathname);
	allSounds.push_back(newsound);
#endif
}

void soundbank_playSound(int id)
{
#ifdef ENABLE_SOUND
	if (!isMute)
		allSounds[id]->play(false);
#endif
}

bool soundbank_isPlayingSound(int id)
{
#ifdef ENABLE_SOUND
	return allSounds[id]->isPlaying();
#else
	return false;
#endif
}


void soundbank_showConsole(void)
{
#ifdef ENABLE_SOUND
    printf("######Beginning of soundbank :\n");
    
    for (std::vector<Sound_OpenAL*>::iterator it = allSounds.begin(); it != allSounds.end(); it++)
        printf("      Sound %s\n",(*it)->isPlaying()?"(playing)":"(stopped)");

    printf("######End of soundbank\n");
#endif
}

void soundbank_freeAll(void)
{
#ifdef ENABLE_SOUND
	for (std::vector<Sound_OpenAL*>::iterator it = allSounds.begin(); it != allSounds.end(); it++)
		delete (*it);
	allSounds.clear();
	deinitSound_OpenAL();
#endif
}

void soundbank_setMute(bool parIsMute)
{
	isMute = parIsMute;
}
