#include "util.h"
#include "player.h"
#include "tableworld.h"
#include "const.h"

bool *table_world;//table des cubes
bool *table_world_is_visited;//table servant a la recherche recursive
int size_x;
int size_y;
int size_z;

ISceneNode **table_cubeSide;
ISceneNode **table_cubeTop;
ISceneNode **table_cubeBottom;

ISceneNode *currentMovingCubeSide=NULL;
ISceneNode *currentMovingCubeTop=NULL;
ISceneNode *currentMovingCubeBottom=NULL;
int directionMovingCube=0;
double destinationMovingCube=0;
int currentMovingCubeX,currentMovingCubeY,currentMovingCubeZ;

static void loadCube(core::vector3df position,
    ITexture* texture_side,
    ITexture* texture_top,
    ITexture* texture_bottom,
    bool is_fog_enable,bool is_glasses_world)
{
    // charger notre cube
	IAnimatedMesh* mesh_side = smgr->getMesh(util_TestExistFile("data/side.3ds"));
	// rcuprer le mesh statique
    ISceneNode * cube_side = smgr->addMeshSceneNode(mesh_side->getMesh(0));
    // lui assigner la texture 
    cube_side->setMaterialTexture(0, texture_side);
    //	activer le brouillard pour cette texture
	if (is_fog_enable) cube_side->setMaterialFlag(EMF_FOG_ENABLE,true);
	
    // charger notre cube
	IAnimatedMesh* mesh_top = smgr->getMesh(util_TestExistFile("data/top.3ds"));
	// rcuprer le mesh statique
    ISceneNode * cube_top = smgr->addMeshSceneNode(mesh_top->getMesh(0));
    // lui assigner la texture 
    cube_top->setMaterialTexture(0, texture_top);
    //	activer le brouillard pour cette texture
  	if (is_fog_enable) cube_top->setMaterialFlag(EMF_FOG_ENABLE,true);
  	//	activer la semi-transparence
  	if (is_glasses_world) cube_top->setMaterialType(EMT_TRANSPARENT_ADD_COLOR);
  	
    // charger notre cube
	IAnimatedMesh* mesh_bottom = smgr->getMesh(util_TestExistFile("data/bottom.3ds"));
	// rcuprer le mesh statique
    ISceneNode * cube_bottom = smgr->addMeshSceneNode(mesh_bottom->getMesh(0));
    // lui assigner la texture 
    cube_bottom->setMaterialTexture(0, texture_bottom);
    //	activer le brouillard pour cette texture
  	if (is_fog_enable) cube_bottom->setMaterialFlag(EMF_FOG_ENABLE,true);
  	//	activer la semi-transparence
  	if (is_glasses_world) cube_bottom->setMaterialType(EMT_TRANSPARENT_ADD_COLOR);
  	//IGUIFont* font = env->getFont("data/fontcourier.bmp");

	cube_top->setScale(vector3df(0.51f,0.51f,0.51f));
	cube_side->setScale(vector3df(0.51f,0.51f,0.51f));
	cube_bottom->setScale(vector3df(0.51f,0.51f,0.51f));
  	
	position=core::vector3df(position.X+CUBE_SIZE/2,position.Y+CUBE_SIZE/2,position.Z+CUBE_SIZE/2);
	// dplacer le cube
	cube_side->setPosition(position);
	cube_top->setPosition(position);
	cube_bottom->setPosition(position);

	cube_side->setMaterialFlag(EMF_LIGHTING, true);
	cube_top->setMaterialFlag(EMF_LIGHTING, true);
	cube_bottom->setMaterialFlag(EMF_LIGHTING, true);
	cube_side->setMaterialFlag(video::EMF_NORMALIZE_NORMALS, true);
	cube_top->setMaterialFlag(video::EMF_NORMALIZE_NORMALS, true);
	cube_bottom->setMaterialFlag(video::EMF_NORMALIZE_NORMALS, true);
	SColor ambientC(255,150,150,150);
	for (unsigned int i = 0; i < cube_side->getMaterialCount(); ++i)
	{
		video::SMaterial &mat = cube_side->getMaterial(i);
		mat.DiffuseColor = ambientC;
		mat.SpecularColor.set(0,0,0,0);
		mat.Shininess = 0;
		mat.AmbientColor = ambientC;
	}
	for (unsigned int i = 0; i < cube_top->getMaterialCount(); ++i)
	{
		video::SMaterial &mat = cube_top->getMaterial(i);
		mat.DiffuseColor = ambientC;
		mat.SpecularColor.set(0,0,0,0);
		mat.Shininess = 0;
		mat.AmbientColor = ambientC;
	}
	for (unsigned int i = 0; i < cube_bottom->getMaterialCount(); ++i)
	{
		video::SMaterial &mat = cube_bottom->getMaterial(i);
		mat.DiffuseColor = ambientC;
		mat.SpecularColor.set(0,0,0,0);
		mat.Shininess = 0;
		mat.AmbientColor = ambientC;
	}
    
    table_cubeSide[(int)((int)(position.X/CUBE_SIZE)+(int)(position.Y/CUBE_SIZE)*size_x+(int)(position.Z/CUBE_SIZE)*size_x*size_y)]= cube_side;
    table_cubeTop[(int)((int)(position.X/CUBE_SIZE)+(int)(position.Y/CUBE_SIZE)*size_x+(int)(position.Z/CUBE_SIZE)*size_x*size_y)] = cube_top;
    table_cubeBottom[(int)((int)(position.X/CUBE_SIZE)+(int)(position.Y/CUBE_SIZE)*size_x+(int)(position.Z/CUBE_SIZE)*size_x*size_y)] = cube_bottom;
}    

void tableworld_QuakeOnAllCubes(double intensity)
{
    if (intensity<=0) return;

    for (int i=0;i<size_x;i++)
        for (int j=0;j<size_y;j++)
            for (int k=0;k<size_z;k++)
            {
                double rand_x=util_Random()-0.5;
                double rand_y=util_Random()-0.5;
                double rand_z=util_Random()-0.5;

                if (tableworld_Get(i,j,k))
                {
                    vector3df pos = table_cubeSide[i+j*size_x+k*size_x*size_y]->getPosition();
                    if (util_Random()>0.9)
                        table_cubeSide[i+j*size_x+k*size_x*size_y]->setPosition(vector3df(i*CUBE_SIZE+CUBE_SIZE/2,j*CUBE_SIZE+CUBE_SIZE/2,k*CUBE_SIZE+CUBE_SIZE/2));
                    else
                        table_cubeSide[i+j*size_x+k*size_x*size_y]->setPosition(vector3df(pos.X+rand_x*intensity,pos.Y+rand_y*intensity,pos.Z+rand_z*intensity));

                    pos = table_cubeSide[i+j*size_x+k*size_x*size_y]->getPosition();
                    table_cubeTop[i+j*size_x+k*size_x*size_y]->setPosition(pos);
                    table_cubeBottom[i+j*size_x+k*size_x*size_y]->setPosition(pos);
                }
            }
}

void tableworld_MoveCube(double distance,Player *lpPlayer,ISceneNode * objective)
{
    if (distance==0) return;
    
    if (directionMovingCube != 0)
    {
        if (currentMovingCubeSide == NULL)
            util_PrintCriticalError("tableworld_MoveCube : pas de cube a bouger");
        
        vector3df pos=currentMovingCubeSide->getPosition();
        
        switch(directionMovingCube)
        {
        case 1:currentMovingCubeSide->setPosition(vector3df(pos.X-distance,pos.Y,pos.Z));
            if (currentMovingCubeSide->getPosition().X<destinationMovingCube) directionMovingCube=0;    
            break;
        case 2:currentMovingCubeSide->setPosition(vector3df(pos.X+distance,pos.Y,pos.Z));
            if (currentMovingCubeSide->getPosition().X>destinationMovingCube) directionMovingCube=0;    
            break;
        case 3:currentMovingCubeSide->setPosition(vector3df(pos.X,pos.Y-distance,pos.Z));
            if (currentMovingCubeSide->getPosition().Y<destinationMovingCube) directionMovingCube=0;    
            break;
        case 4:currentMovingCubeSide->setPosition(vector3df(pos.X,pos.Y+distance,pos.Z));
            if (currentMovingCubeSide->getPosition().Y>destinationMovingCube) directionMovingCube=0;    
            break;
        case 5:currentMovingCubeSide->setPosition(vector3df(pos.X,pos.Y,pos.Z-distance));
            if (currentMovingCubeSide->getPosition().Z<destinationMovingCube) directionMovingCube=0;    
            break;
        case 6:currentMovingCubeSide->setPosition(vector3df(pos.X,pos.Y,pos.Z+distance));
            if (currentMovingCubeSide->getPosition().Z>destinationMovingCube) directionMovingCube=0;    
            break;
        }
        
        if (directionMovingCube!=0)
        {
            vector3df v=currentMovingCubeSide->getPosition();
            currentMovingCubeTop->setPosition(v);
            currentMovingCubeBottom->setPosition(v);
        }
        else
        {   
            currentMovingCubeSide->setPosition(vector3df(
                currentMovingCubeX*CUBE_SIZE+CUBE_SIZE/2,
                currentMovingCubeY*CUBE_SIZE+CUBE_SIZE/2,
                currentMovingCubeZ*CUBE_SIZE+CUBE_SIZE/2));
            
            vector3df v=currentMovingCubeSide->getPosition();
            currentMovingCubeTop->setPosition(v);
            currentMovingCubeBottom->setPosition(v);
            tableworld_Set(currentMovingCubeX,currentMovingCubeY,currentMovingCubeZ,true);
            
            table_cubeSide[currentMovingCubeX+currentMovingCubeY*size_x+currentMovingCubeZ*size_x*size_y] = currentMovingCubeSide;
            table_cubeTop[currentMovingCubeX+currentMovingCubeY*size_x+currentMovingCubeZ*size_x*size_y] = currentMovingCubeTop;
            table_cubeBottom[currentMovingCubeX+currentMovingCubeY*size_x+currentMovingCubeZ*size_x*size_y] = currentMovingCubeBottom;
        }
    }
    
    if (directionMovingCube == 0)
    {   
        int x,y,z,x2,y2,z2,dir;
        do 
        {
            x=(int)(util_Random()*size_x);
            y=(int)(util_Random()*size_y);
            z=(int)(util_Random()*size_z);
            dir=(int)(util_Random()*6);
            switch(dir)
            {
                case 0:x2=x-1;y2=y;z2=z;directionMovingCube=1;destinationMovingCube=x2*CUBE_SIZE+CUBE_SIZE/2;break;
                case 1:x2=x+1;y2=y;z2=z;directionMovingCube=2;destinationMovingCube=x2*CUBE_SIZE+CUBE_SIZE/2;break;
                case 2:x2=x;y2=y-1;z2=z;directionMovingCube=3;destinationMovingCube=y2*CUBE_SIZE+CUBE_SIZE/2;break;
                case 3:x2=x;y2=y+1;z2=z;directionMovingCube=4;destinationMovingCube=y2*CUBE_SIZE+CUBE_SIZE/2;break;
                case 4:x2=x;y2=y;z2=z-1;directionMovingCube=5;destinationMovingCube=z2*CUBE_SIZE+CUBE_SIZE/2;break;
                case 5:x2=x;y2=y;z2=z+1;directionMovingCube=6;destinationMovingCube=z2*CUBE_SIZE+CUBE_SIZE/2;break;
            }
        }
        while(!(tableworld_Get(x,y,z) && !tableworld_Get(x2,y2,z2)
            && x2>=0 && x2<size_x && y2>=0 && y2<size_y && z2>=0 && z2<size_z));
        //printf("ok %d - %d %d %d - %d %d %d\n",tableworld_Get(x,y,z),x,y,z,x2,y2,z2);
        currentMovingCubeSide = table_cubeSide[x+y*size_x+z*size_x*size_y];
        currentMovingCubeTop = table_cubeTop[x+y*size_x+z*size_x*size_y];
        currentMovingCubeBottom = table_cubeBottom[x+y*size_x+z*size_x*size_y];
        
        vector3df v = objective->getPosition();

        double distanceCubePlayer = util_GetDistance(currentMovingCubeSide->getPosition(),lpPlayer->GetPosition());
        //printf("dist %lf\n",distanceCubePlayer);
        if (distanceCubePlayer<CUBE_SIZE*3 || distanceCubePlayer>CUBE_SIZE*5
            || (x2==(int)(v.X/CUBE_SIZE) && y2==(int)(v.Y/CUBE_SIZE) && z2==(int)(v.Z/CUBE_SIZE)))
        {   
            directionMovingCube=0;
            //printf("removed\n");
            return tableworld_MoveCube(distance,lpPlayer,objective);
        }
        
        currentMovingCubeX=x2;
        currentMovingCubeY=y2;
        currentMovingCubeZ=z2;
        
        tableworld_Set(x,y,z,false);
    }
}

void tableworld_LoadCubes(
    bool is_fog_enable,bool is_glasses_world,
    ITexture* texture_side,ITexture* texture_top,ITexture* texture_bottom)
{
	//cree les cubes
    for (int i=0;i<size_x;i++)
        for (int j=0;j<size_y;j++)
            for (int k=0;k<size_z;k++)
            {
                if (tableworld_Get(i,j,k)) //s'il doit y avoir un cube
                {
                    //le creer
                    loadCube(vector3df(i*CUBE_SIZE,j*CUBE_SIZE,k*CUBE_SIZE),
                                        texture_side,texture_top,texture_bottom,
                                        is_fog_enable,is_glasses_world);
                }    
            }    
}

bool tableworld_RecursiveTest(int x,int y,int z,int objective_x,int objective_y,int objective_z)
{
    table_world_is_visited[x+y*size_x+z*size_x*size_y]=true;

    if (x==objective_x && y==objective_y && z==objective_z) //found the objective
        return true;
    
    bool tmp1=false,tmp2=false,tmp3=false,tmp4=false,tmp5=false,tmp6=false;
    
    if (x-1>0 && !tableworld_Get(x-1,y,z) 
        && !table_world_is_visited[(x-1)+y*size_x+z*size_x*size_y])
        tmp1 = tableworld_RecursiveTest(x-1,y,z,objective_x,objective_y,objective_z);

    if (!tmp1 && x+1<size_x && !tableworld_Get(x+1,y,z)
        && !table_world_is_visited[(x+1)+y*size_x+z*size_x*size_y])
        tmp2 = tableworld_RecursiveTest(x+1,y,z,objective_x,objective_y,objective_z);

    if (!tmp1 && !tmp2 && y-1>0 && !tableworld_Get(x,y-1,z)
        && !table_world_is_visited[x+(y-1)*size_x+z*size_x*size_y])
        tmp3 = tableworld_RecursiveTest(x,y-1,z,objective_x,objective_y,objective_z);

    if (!tmp1 && !tmp2 && !tmp3 && y+1<size_y && !tableworld_Get(x,y+1,z)
        && !table_world_is_visited[x+(y+1)*size_x+z*size_x*size_y])
        tmp4 = tableworld_RecursiveTest(x,y+1,z,objective_x,objective_y,objective_z);

    if (!tmp1 && !tmp2 && !tmp3 && !tmp4 && z-1>0 && !tableworld_Get(x,y,z-1)
        && !table_world_is_visited[x+y*size_x+(z-1)*size_x*size_y])
        tmp5 = tableworld_RecursiveTest(x,y,z-1,objective_x,objective_y,objective_z);

    if (!tmp1 && !tmp2 && !tmp3 && !tmp4 && !tmp5 && z+1<size_z && !tableworld_Get(x,y,z+1)
        && !table_world_is_visited[x+y*size_x+(z+1)*size_x*size_y])
        tmp6 = tableworld_RecursiveTest(x,y,z+1,objective_x,objective_y,objective_z);
    
    return (tmp1 || tmp2 || tmp3 || tmp4 || tmp5 || tmp6);
}

bool tableworld_TestMazeIsSolvable(vector3df initial_pos,vector3df objective_pos)
{
    return tableworld_RecursiveTest(
        (int)(initial_pos.X/CUBE_SIZE),
        (int)(initial_pos.Y/CUBE_SIZE),
        (int)(initial_pos.Z/CUBE_SIZE),
        (int)(objective_pos.X/CUBE_SIZE),
        (int)(objective_pos.Y/CUBE_SIZE),
        (int)(objective_pos.Z/CUBE_SIZE));
}

void tableworld_Create(int size_x_,int size_y_,int size_z_,double probability_cube)
{
    directionMovingCube=0;

    size_x=size_x_;
    size_y=size_y_;
    size_z=size_z_;
    
    table_world=new bool[size_x*size_y*size_z];
    table_world_is_visited=new bool[size_x*size_y*size_z];
    table_cubeSide=new ISceneNode *[size_x*size_y*size_z];
    table_cubeTop=new ISceneNode *[size_x*size_y*size_z];
    table_cubeBottom=new ISceneNode *[size_x*size_y*size_z];

    //cree les cubes et remplit la table de collision
    for (int i=0;i<size_x;i++)
        for (int j=0;j<size_y;j++)
            for (int k=0;k<size_z;k++)
            {
                if (i==0 && j==0 && k==0) //evite qu'il n'y ai pas de cube
                    tableworld_Set(i,j,k,true);
                else if (i==0 && j==1 && k==0) //evite qu'il n'y ai que des cubes
                    tableworld_Set(i,j,k,false);
                else if (util_Random()>1-probability_cube) //cube
                    tableworld_Set(i,j,k,true);
                else //pas de cube
                    tableworld_Set(i,j,k,false);
                 
                table_world_is_visited[i+j*size_x+k*size_x*size_y]=false;
            }    
}

void tableworld_Set(int x,int y,int z,bool value)
{
    if (x<0 || y<0 || z<0 || x>=size_x || y>=size_y || z>=size_z) return;
    
    table_world[x+y*size_x+z*size_x*size_y]=value;
}    

bool tableworld_Get(int x,int y,int z)
{
    if (x<0 || y<0 || z<0 || x>=size_x || z>=size_z)
        return false;
    if (y>=size_y) 
        return true;
    return table_world[x+y*size_x+z*size_x*size_y];
}    

bool tableworld_GetWithPos(core::vector3df v)
{
    if (v.X<0) return false;
    if (v.Y<0) return false;
    if (v.Z<0) return false;
    
    return tableworld_Get((int)(v.X/CUBE_SIZE),(int)(v.Y/CUBE_SIZE),(int)(v.Z/CUBE_SIZE));
}    

void tableworld_Destroy(void)
{
    delete [] table_world;
    delete [] table_world_is_visited;
    delete [] table_cubeSide;
    delete [] table_cubeTop;
    delete [] table_cubeBottom;
}    


vector3df tableworld_GetRandomPosition(void)
{
    int x=(int)((double)util_Random()*(double)size_x);
    int y=(int)((double)util_Random()*((double)size_y-1)+1);
    int z=(int)((double)util_Random()*(double)size_z);

    if (tableworld_Get(x,y,z)==true)
        return tableworld_GetRandomPosition();

    int k=y-1;
    while (tableworld_Get(x,k,z)==false && k>=0)
        k--;
    
    if (k<0)//try again
        return tableworld_GetRandomPosition();
    else
        return vector3df(x*CUBE_SIZE+CUBE_SIZE/2,(k+1)*CUBE_SIZE+CUBE_SIZE/2,z*CUBE_SIZE+CUBE_SIZE/2);
        
}    

