#include <stdio.h>
#include "util.h"

#include "world.h"
#include "settings.h"
#include "player.h"
#include "tableworld.h"
#include "const.h"

static Player *lpPlayer;
ICameraSceneNode* camera;

ISceneNode * objective;
ISceneNode * objective2;

IGUIImage* img_compass_player;
ITexture* texture_compass_player;
ITexture* texture_side=NULL;
ITexture* texture_top=NULL;
ITexture* texture_bottom=NULL;

int remainingTime=0;
bool isInfiniteTime;

static ISceneNode *addSceneNodeInWorld(const char *mesh_path,
    const char *texture_path,bool is_fog,core::vector3df pos)
{   
    if (mesh_path == NULL)
        util_PrintCriticalError("Object don't has any mesh");
    else
        util_TestExistFile(mesh_path);

  	IAnimatedMesh* n_mesh = smgr->getMesh(mesh_path);
   	// rcuprer le mesh statique
    ISceneNode *n = smgr->addMeshSceneNode(n_mesh->getMesh(0));
    // lui assigner la texture qu'on charge  la vole
    if (texture_path != NULL)
        n->setMaterialTexture(0, driver->getTexture(texture_path));
    else
        util_TestExistFile(texture_path);
    
    // supprimer la gestion de l'clairage sur le cube
  	n->setMaterialFlag(EMF_LIGHTING, false);
  	//	activer le brouillard pour cette texture
   	if (is_fog) n->setMaterialFlag(EMF_FOG_ENABLE,true);
  	//positionner l'objet
   	n->setPosition(pos);
   	  	
  	return n;
}    
static void world_loadAutoConfiguration(int level)
{
    settings_SetValue("size_x_world",7.0);
    settings_SetValue("size_y_world",7.0);
    settings_SetValue("size_z_world",7.0);
    settings_SetValue("probability_cube",0.5);
    settings_SetValue("is_subjective",1.0);
    settings_SetValue("is_glasses_world",0.0);
    settings_SetValue("is_fog_enable",0.0);
    settings_SetValue("quake_intensity",0.0);
    settings_SetValue("time_counter_speed",(1000-level*50)/settings_GetValue("difficulty_level"));
    
    switch(level)
    {
        case 1:settings_SetValue("moving_cubes",0.0);break;
        case 2:
            settings_SetValue("is_glasses_world",1.0);
            settings_SetValue("size_x_world",4.0);
            settings_SetValue("size_y_world",20.0);
            settings_SetValue("size_z_world",4.0);
            break;
        case 3:
            settings_SetValue("size_x_world",12.0);
            settings_SetValue("size_y_world",12.0);
            settings_SetValue("size_z_world",12.0);
            settings_SetValue("probability_cube",0.15);
            settings_SetValue("is_subjective",0.0);
            break;
        case 4:
            settings_SetValue("size_x_world",18.0);
            settings_SetValue("size_y_world",18.0);
            settings_SetValue("size_z_world",18.0);
            settings_SetValue("probability_cube",0.02);
            break;
        case 5:
            settings_SetValue("size_x_world",20.0);
            settings_SetValue("size_y_world",20.0);
            settings_SetValue("size_z_world",2.0);
            settings_SetValue("probability_cube",0.5);
            break;
        case 6:
            settings_SetValue("size_x_world",15.0);
            settings_SetValue("size_y_world",3.0);
            settings_SetValue("size_z_world",15.0);
            settings_SetValue("probability_cube",0.3);
            break;
        case 7:
            settings_SetValue("is_fog_enable",1.0);
            settings_SetValue("size_x_world",3.0);
            settings_SetValue("size_y_world",10.0);
            settings_SetValue("size_z_world",10.0);
            settings_SetValue("probability_cube",0.7);
            break;
        case 8:
            settings_SetValue("size_x_world",8.0);
            settings_SetValue("size_y_world",8.0);
            settings_SetValue("size_z_world",8.0);
            settings_SetValue("quake_intensity",0.5);
            settings_SetValue("probability_cube",0.7);
            break;
        case 9:
	    settings_SetValue("is_glasses_world",1.0);
            settings_SetValue("is_fog_enable",1.0);
            settings_SetValue("size_x_world",18.0);
            settings_SetValue("size_y_world",3.0);
            settings_SetValue("size_z_world",18.0);
            settings_SetValue("probability_cube",0.2);
            break;
        case 10:
            settings_SetValue("size_x_world",8.0);
            settings_SetValue("size_y_world",8.0);
            settings_SetValue("size_z_world",8.0);
            break;
        case 11:
            settings_SetValue("size_x_world",7.0);
            settings_SetValue("size_y_world",7.0);
            settings_SetValue("size_z_world",7.0);
            settings_SetValue("probability_cube",0.2);
            settings_SetValue("is_subjective",0.0);
            settings_SetValue("time_counter_speed",settings_GetValue("time_counter_speed")*2/3);
            settings_SetValue("is_glasses_world",1.0);
            break;
        default:fprintf(stderr,"WARNING : level unknown for autoConfiguration");break;
    }
}


void world_Load(int level,IGUIStaticText *loading_status_text)
{
    printf("Creating level...\n");

    if ((int)settings_GetValue("is_autoconfigure"))
        world_loadAutoConfiguration(level);
    
    char *str1=new char[40+(int)(level/10)];
    char *str2=new char[40+(int)(level/10)];
    char *str3=new char[40+(int)(level/10)];
    char *str4=new char[40+(int)(level/10)];
    char *str5=new char[40+(int)(level/10)];
    char *str6=new char[40+(int)(level/10)];
    //sprintf(str1,"data/%d/side.bmp",level);
	sprintf(str1,"data/%d/side.bmp",level);
    sprintf(str2,"data/%d/top.bmp",level);
    sprintf(str3,"data/%d/bottom.bmp",level);
    
    loading_status_text->setText(L". Loading world textures");
    driver->beginScene(true, true, video::SColor(0,0,0,0));env->drawAll();driver->endScene();
    
    texture_side=driver->getTexture(util_TestExistFile(str1));
    texture_top=driver->getTexture(util_TestExistFile(str2));
    texture_bottom=driver->getTexture(util_TestExistFile(str3));
    
    /************************
    cr la table du monde
    ************************/
    
    loading_status_text->setText(L".. Creating world");
    driver->beginScene(true, true, video::SColor(0,0,0,0));env->drawAll();driver->endScene();
    
    //get informations - recupere des infos
    int size_x_world=(int)settings_GetValue("size_x_world");
    int size_y_world=(int)settings_GetValue("size_y_world");
    int size_z_world=(int)settings_GetValue("size_z_world");
    
    bool is_subjective=(bool)settings_GetValue("is_subjective");
    bool is_fog_enable=(bool)settings_GetValue("is_fog_enable");

    //cree et remplit la table de collision
    tableworld_Create(size_x_world,size_y_world,size_z_world,settings_GetValue("probability_cube"));

    /************************
    chargement du joueur
    ************************/
    
    loading_status_text->setText(L"... Loading player (might be long)");
    driver->beginScene(true, true, video::SColor(0,0,0,0));env->drawAll();driver->endScene();
    
    //create the player - cree le joueur
    if (is_subjective)
        lpPlayer=new Player("data/top.3ds",NULL);
    else
        lpPlayer=new Player("data/blob.3ds",NULL);
    
    /************************
    chargement du panneau fin
    ************************/
    
	loading_status_text->setText(L".... Loading objective");
	driver->beginScene(true, true, video::SColor(0,0,0,0));env->drawAll();driver->endScene();
	
	objective2 = addSceneNodeInWorld("data/cube.3ds","data/objectiv.png",
	  	is_fog_enable,tableworld_GetRandomPosition());
	objective2->setMaterialType(video::EMT_TRANSPARENT_ALPHA_CHANNEL);
	objective2->setScale(vector3df(2.f,2.f,2.f));

	//create end - creer le panneau fin
  	objective = addSceneNodeInWorld("data/cube.3ds","data/objectiv2.png",
	  	is_fog_enable,tableworld_GetRandomPosition());
	//objective->setMaterialType(EMT_TRANSPARENT_ADD_COLOR);
	objective->setMaterialType(video::EMT_TRANSPARENT_ALPHA_CHANNEL);
	//objective_top->setMaterialType(EMT_TRANSPARENT_ADD_COLOR);
	objective->setMaterialFlag(EMF_LIGHTING, true);

	objective->setScale(vector3df(0.51f,0.51f,0.51f));
	objective2->setScale(vector3df(0.51f,0.51f,0.51f));
	objective2->setPosition(objective->getPosition());
	
    /************************
    test du monde
    ************************/
    
    loading_status_text->setText(L"..... Testing maze");
    driver->beginScene(true, true, video::SColor(0,0,0,0));env->drawAll();driver->endScene();
    if (lpPlayer->GetPosition()==objective->getPosition()
        || !tableworld_TestMazeIsSolvable(objective->getPosition(),lpPlayer->GetPosition()))
    //if the maze is unsolvable - si le laby est irresolvable 
    {
        //reload all the level - recharge tout le niveau
        world_Destroy();
        world_Load(level,loading_status_text);
        return;
    }
    
    /************************
    chargement du monde
    ************************/
    
    loading_status_text->setText(L"...... Loading world meshes and loading cubes (might be long)");
    driver->beginScene(true, true, video::SColor(0,0,0,0));env->drawAll();driver->endScene();
    
    //load textures - charge les textures
    //cree les cubes
    tableworld_LoadCubes(
    (bool)settings_GetValue("is_fog_enable"),(bool)settings_GetValue("is_glasses_world"),
        texture_side,texture_top,texture_bottom);
    
	sprintf(str1,"data/%d/sky_top.bmp",level);
	sprintf(str2,"data/%d/sky_bottom.bmp",level);
	sprintf(str3,"data/%d/sky_left.bmp",level);
	sprintf(str4,"data/%d/sky_right.bmp",level);
	sprintf(str5,"data/%d/sky_front.bmp",level);
	sprintf(str6,"data/%d/sky_back.bmp",level);
	
    /************************
    chargement de la skybox, du plafond, de la camera 
    ************************/
    
	loading_status_text->setText(L"....... Loading skybox textures, ceil and camera");
	driver->beginScene(true, true, video::SColor(0,0,0,0));env->drawAll();driver->endScene();
	
	//cree la skybox
	ISceneNode* skybox = smgr->addSkyBoxSceneNode(
		driver->getTexture(util_TestExistFile(str1)),
		driver->getTexture(util_TestExistFile(str2)),
		driver->getTexture(util_TestExistFile(str3)),
		driver->getTexture(util_TestExistFile(str4)),
		driver->getTexture(util_TestExistFile(str5)),
		driver->getTexture(util_TestExistFile(str6)));
    
	delete [] str1;
	delete [] str2;
	delete [] str3;
	delete [] str4;
	delete [] str5;
	delete [] str6;
	    
    //cree le plafond
    for (int i=0;i<size_x_world;i++)
        for (int k=0;k<size_z_world;k++)
        {
            IAnimatedMesh* mesh_ceiling = smgr->getMesh(util_TestExistFile("data/bottom.3ds"));
        	// rcuprer le mesh statique
            ISceneNode * ceiling = smgr->addMeshSceneNode(mesh_ceiling->getMesh(0));
            // lui assigner la texture 
			ceiling->setScale(vector3df(0.51f,0.51f,0.51f));
            ceiling->setMaterialTexture(0, texture_top);
            // supprimer la gestion de l'clairage 
        	ceiling->setMaterialFlag(EMF_LIGHTING, false);
        	//	activer le brouillard pour cette texture
          	if (is_fog_enable) ceiling->setMaterialFlag(EMF_FOG_ENABLE,true);
          	//positionner le plafond
          	ceiling->setPosition(vector3df((i+.5)*CUBE_SIZE,(size_y_world+.5)*CUBE_SIZE,(k+.5)*CUBE_SIZE));
          	ceiling->setMaterialType(EMT_TRANSPARENT_ADD_COLOR);
       	}    
       	
    ILightSceneNode *light=smgr->addLightSceneNode(0,objective->getPosition(),SColorf(1.0f, 1.0f, 1.0f), 2000.0f);
	light->setPosition(vector3df(0,250,0));
    
    camera = smgr->addCameraSceneNode();
    smgr->setActiveCamera(camera);
//    camera->setPosition(vector3df(100,100,0));
//    camera->setRotation(vector3df(100,0,20));
//    camera->setTarget(vector3df(0.0,9.8,80.0));
    camera->setInputReceiverEnabled(0);
	
    if (settings_GetValue("time_counter_speed")>0.0 || (int)settings_GetValue("is_autoconfigure"))
    	isInfiniteTime=false;
    else
	isInfiniteTime=true;
	
    remainingTime = lpPlayer->GetDistanceFromObjective(objective)*(int)settings_GetValue("time_counter_speed") + util_GetTimerTime();

	smgr->setAmbientLight(video::SColor(255,255,255,255));

    //    if (is_subjective)
    //    camera->setFOV(PI / 5.5f);//PI / 3.5f
    //ICameraSceneNode* camera2 = smgr->addCameraSceneNodeFPS();
    printf("done.\n");
}    

int world_Play(void)
{
    if (mouseWheel<1) mouseWheel=1;
    if (mouseWheel>16) mouseWheel=16;
    camera->setFOV(1.7 * PI / (10.0-mouseWheel/2.0));
    
    tableworld_QuakeOnAllCubes((double)((double)30/(double)util_GetFPS()*settings_GetValue("quake_intensity")));
    
    if (settings_GetValue("moving_cubes"))
        tableworld_MoveCube((double)((double)30/(double)util_GetFPS()*1),lpPlayer,objective);
	return lpPlayer->Play(camera,objective);
}    

void world_Destroy(void)
{
    printf("Removing level...\n");
    //driver->removeAllTextures();

    if (texture_side) driver->removeTexture(texture_side);
    if (texture_top) driver->removeTexture(texture_top);
    if (texture_bottom) driver->removeTexture(texture_bottom);
  
    
    tableworld_Destroy();
    delete lpPlayer;
    smgr->clear();
    printf("done.\n");
}

int world_GetRemainingTime(void)
{
    if (isInfiniteTime) return -1000;
    else return remainingTime-util_GetTimerTime();
}

void world_CustomDraw(void)
{
	objective2->setVisible(true);
	objective2->setMaterialFlag(EMF_ZBUFFER, false);
	objective2->render();
	//objective->setVisible(false);
}